unit UGoal2DRotate;

interface

uses UGoal, UPlayer2D;


type TGoal2DRotate = class(TGoal)
  private
    CX, CY: double;
  protected
  public
    constructor create(C, X, Y: double);
    procedure Draw; override;
    procedure CalcScore(FireAt: TGoalFire); override;
  end;


implementation

uses Math, Graphics, UUtils, UTrack2D;


constructor TGoal2dRotate.create(C, X, Y: double);
begin
  inherited create(c);
  fDrawable:= true;
  CX:= X;
  CY:= Y;
  Strength:= C;
  fDisplayName:= 'Rotate CCW';
end;

procedure TGoal2dRotate.CalcScore(FireAt: TGoalFire);
var
  A1, A2: double;
  p: TPlayer2D;
begin
  inherited;
  p:= TPLayer2D(Player);
  if p.Dead then exit;
  if (FireAt = gfStep) then begin
    // If the player was N degrees form origin, and now is N+A, add this "A":
    A1:= ArcTan2(-(p.PosY-CY), (p.PosX-CX)) / PI*180;
    while (A1<0) do A1:= A1+360;
    A2:= ArcTan2(-(p.OldPosY-CY), (p.OldPosX-CX)) / PI*180;
    while (A2<0) do A2:= A2+360;
    Score:= (A2-A1);
    if Score > +180 then
      Score:= Score - 360;
    if Score < -180 then
      Score:= Score + 360;
    Score:= Max(0, Score);
  end;
end;

procedure TGoal2dRotate.Draw;
var
  p: TPlayer2D;
  C: TCanvas;
begin
  p:= TPlayer2D(Player);
  if not p.Imagining then begin
    C:= TTrack2D(p.Track).Img.Canvas;
    // A circle on the center:
    C.Brush.Color:= clCream;
    C.Pen.Color:= clBlack;
    C.Ellipse(round(CX-5), round(CY-5), round(CX+5), round(CY+5));
    C.Brush.Color:= clRed;
    C.Ellipse(round(CX-2), round(CY-2), round(CX+2), round(CY+2));
    // Line to the player being "motivated":
    C.Pen.Color:= RGBToColor(0, 255, 0);
    C.MoveTo(round(CX), round(CY));
    C.LineTo(round(p.PosX), round(p.PosY));
  end;
end;

end.
