unit UGoal2DStorage;

// Players with load will drop it here when touch and the amount of load stored
// will be your score.

interface

uses UGoal, UPlayer2D;


type TGoal2DStorage = class(TGoal)
  private
    // Position and size of the storage facility:
    X, Y, Size: integer;
    // Capacity, how much load it can store:
    MaxCapacity: double;
    // Actual amount of load in the storage:
    Used: double;
  protected
  public
    constructor create(c: double; vx, vy, vsize, vmax: integer);
    procedure NeutralDraw; override;
    procedure CalcScore(FireAt: TGoalFire); override;
  end;


implementation

uses UTrack2D, Graphics, ExtCtrls, Math;


constructor TGoal2DStorage.create(c: double; vx, vy, vsize, vmax: integer);
begin
  inherited create(c);
  X:= vx;
  Y:= vy;
  MaxCapacity:= vmax;
  Size:= Max(1, vsize);
  fDisplayName:= 'Store drops';
end;

procedure TGoal2DStorage.CalcScore(FireAt: TGoalFire);
var
  p: TPlayer2D;
  OldUsed: double;
  Gain: double;
begin
  inherited;
  p:= TPLayer2D(player);
  if p.Dead then exit;
  if (FireAt = gfStep) then begin
    // When you come near, your energy is downloaded ;-) down to 10%:
    if (Used < MaxCapacity) and
       (abs(p.PosX-X)<(p.Size+Size)) and
       (abs(p.PosY-Y)<(p.Size+Size)) then begin
      // How much of the load can I store?
      OldUsed:= Used;
      Used:= Used + Max(0, p.Energy-10); // Max value will limit the load I can add to the store
      Gain:= Used - OldUsed;
      Used:= OldUsed;
      // Gained load is what scores and what you take off from the drop:
      Score:= Score + Gain;
      // If it is real, pass the load from player to store:
      if not p.Imagining then begin
        p.Energy:= p.Energy - Gain;
        Used:= Used + Gain;
      end;
    end;
  end;
end;

procedure TGoal2dStorage.NeutralDraw;
var
  p: TPlayer2D;
  C: TCanvas;
begin
  if not Assigned(Player) then exit;
  p:= TPlayer2D(player);
  C:= TTrack2D(p.Track).Img.Canvas;
  // A square half filled will do as a storage:
  C.Pen.Color:= clBlue;
  C.Brush.Style:= bsClear;
  C.Rectangle(round(X-Size), round(Y-Size), round(X+Size), round(Y+Size));
  // How full is it?
  C.Pen.Color:= clBlack;
  C.Brush.Color:= clAqua;
  C.Rectangle(round(X-Size), round(Y+Size - ceil(2*Size*(Used/MaxCapacity))), round(X+Size), round(Y+Size));
end;

end.
