unit UGoal2DSweets;

// A lot of energy drops that jump into the player's mouth when touched. The more
// you eat, the more it score, so when you are full it won't score anymore.
// Restricted for Rockets only to simulate evolution: Karts eat rockets...

interface

uses UGoal, UPlayer2D;


type TGoal2DSweets = class(TGoal)
  private
    FirstRun: boolean;
    X:  array [0..99] of integer;
    Y:  array [0..99] of integer;
    SR: array [0..99] of integer; // Sweet points scores (R-eal one)
    SI: array [0..99] of integer; // Points copy to use while I-maginig
    StillAlive: integer;
    Continuous: boolean;
    procedure AddOneDrop(i: integer);
    procedure AddAllDrops;
  protected
    procedure CalcScore(FireAt: TGoalFire); override;
  public
    constructor create(c: double); override;
    procedure ResetImaginaryState; override;
    procedure NeutralDraw; override;
  end;

const
  NDrops: integer =   100;
  MinDrops: integer = 20;


implementation

uses UTrack2D, UTrack, URocket, Graphics, ExtCtrls, Math;


constructor TGoal2DSweets.create(c: double);
begin
  inherited;
  FirstRun:= true;
  //SoonerIsBetter:= true;
  Continuous:= true;
  fDisplayName:= 'Get drops';
end;

procedure TGoal2DSweets.AddOneDrop(i: integer);
var
  Img: TImage;
  p: TPlayer2D;
  IsInside: boolean;
begin
  // Only redefine empty drops:
  if (SR[i]>0) then exit;
  p:= TPLayer2D(player);
  Img:= TTrack2D(p.Track).Img;
  // Define size of the drop:
  SR[i]:= 2+round(random*4);
  SI[i]:= SR[i];
  // Define location of the drop:
  repeat
    X[i]:= round((0.1 + random*0.8)*Img.Width);
    Y[i]:= round((0.1 + random*0.8)*Img.Height);
    IsInside:= TTrack2D(p.Track).IsInsideTrack(X[i], Y[i]) and
             TTrack2D(p.Track).IsInsideTrack(round(X[i]+p.Size*1.5), Y[i]) and
             TTrack2D(p.Track).IsInsideTrack(round(X[i]-p.Size*1.5), Y[i]) and
             TTrack2D(p.Track).IsInsideTrack(X[i], round(Y[i]+p.Size*1.5)) and
             TTrack2D(p.Track).IsInsideTrack(X[i], round(Y[i]-p.Size*1.5));
  until IsInside;
end;

procedure TGoal2DSweets.AddAllDrops;
var i: integer;
begin
  for i:= 0 to NDrops-1 do
    AddOneDrop(i);
  FirstRun:= false;
  StillAlive:= NDrops; // If it reach MinDrops it will add more
end;

procedure TGoal2DSweets.CalcScore(FireAt: TGoalFire);
var
  p: TPlayer2D;
  i: integer;
  OldEnergy, Gain: double;
  S: integer;
const
  // How much energy a size 1 drop contains:
  FaceValue: integer = 2;
begin
  inherited;
  p:= TPLayer2D(player);
  if p.Dead then exit;

  if TTrack(p.Track).Evolution and not (p is TRocket) then exit;

  if (FireAt = gfStep) then begin
    // Add 100 sweets on the track if not already done:
    if FirstRun then
      AddAllDrops;
    // When you step over a drop, the "energy" of the drop get added:
    for i:= 0 to NDrops-1 do begin
      if p.Imagining then
        S:= SI[i]
      else
        S:= SR[i];
      if (S>0) and (abs(p.PosX-X[i])<p.Size) and (abs(p.PosY-Y[i])<p.Size) then begin
        // Drops gives energy, so if you are not full, your energy will increase
        OldEnergy:= p.Energy;
        p.Energy:= p.Energy + FaceValue;
        Gain:= (p.Energy - OldEnergy) / FaceValue;
        // NOTE: This goal gets energy, scoring is done with a GoalEnergy.
        // Optionally you can give a extra scoring, but it is not mandatory.:
        if (Gain<>0) then begin
          // Scoring is not mandatory, getting energy should be ok by itself.
          Score:= Score + Gain;
          // If it is real life, kill this sweet:
          if p.Imagining then begin
            SI[i]:= Max(0, round(SI[i]-Gain));
          end else begin
            SR[i]:= Max(0, round(SR[i]-Gain));
            if (SR[i]=0) then begin
              if Continuous then
                AddOneDrop(i)
              else
                dec(StillAlive);
            end;
            // No more drops? Create more!
            if (StillAlive<=MinDrops) then
              FirstRun:= true;
          end;
        end;
      end;
    end;
  end;
end;

procedure TGoal2dSweets.NeutralDraw;
var
  p: TPlayer2D;
  C: TCanvas;
  i, r: integer;
begin
  if FirstRun or (not Assigned(player)) then exit;
  p:= TPlayer2D(player); // I still have it from last player's CalScore
  if p.Imagining then exit;
  C:= TTrack2D(p.Track).Img.Canvas;
  C.Brush.Color:= clCream;
  C.Pen.Color:= clGreen;
  for i:= 0 to NDrops-1 do begin
    if (SR[i] > 0) then begin
      r:= (SR[i]+1);
      C.Ellipse(round(X[i]-r), round(Y[i]-r), round(X[i]+r), round(Y[i]+r));
    end;
  end;
end;

// Update the imaginary copy of the scores:
procedure TGoal2dSweets.ResetImaginaryState;
var i: integer;
begin
  for i:= 0 to High(SR) do
    SI[i]:= SR[i];
end;


end.
