unit UGoalTimer;

interface

uses UPlayer, UTrack, UGoal;

type TGoalTimer = class(TGoal)
  private
    fTimeToLive: integer; // Minimum milliseconds to keep alive (usually 500)
    fAfraid: double;      // How scary is it for you to die (tipically +500)
  protected
  public
    constructor create(c: double); override;
    procedure CalcScore(FireAt: TGoalFire); override;
    procedure Draw; override;
    property  Afraid: double read fAfraid write fAfraid;
    property  TimeToLive: integer read fTimeToLive write fTimeToLive;
  end;


implementation

uses Graphics, Math, UState, UPlayer2D, UTrack2D;


constructor TGoalTimer.create(c: double);
begin
  inherited;
  fDrawable:= true;
  fDrawOnImagining:= true;
  TimeToLive:= 700; // Minimum time to keep alive (700 ms default)
  Afraid:=     200; // How scary (negative scoring) is to die? (200 default)
  fDisplayName:= 'Time to live';
end;

procedure TGoalTimer.CalcScore(FireAt: TGoalFire);
var
  p: TPlayer;
  MaxTime: integer;
begin
  inherited;
  if (FireAt = gfStep) then begin
    if (Strength>0) and (Afraid>0) then begin
      p:= TPlayer(Player);
      // If you just died now and you were imagining a future
      if (p.Dead and not p.OldDead) and p.Imagining then begin
        MaxTime:= TTrack(p.Track).FutureMSeconds;
        //MaxTime:= 2*TimeToLive;
        if (p.State.TimePassed < MaxTime) then
          Score:= -Afraid * Min(1.0, Power((MaxTime-p.State.TimePassed)/(1.0*(MaxTime-TimeToLive)) , 4 ));
      end;
    end;
  end;
end;

procedure TGoalTimer.Draw;
var
  p: TPlayer2D;
  C: TCanvas;
  r: double;
begin
  //if (TimeLeft < 1) and (Player is TPlayer2D) then begin
  if Score < -1 then begin
    p:= TPlayer2D(player);
    C:= TTrack2D(p.Track).Img.Canvas;
    // Mark points where fear to die switched on:
    C.Pen.Color:= clRed;
    //r:= 5*(1-TimeLeft);
    r:= 3*(-Score/Afraid);
    C.Brush.Color:= p.Color;
    C.Pen.Color:= clBlack;
    C.Ellipse(round(p.PosX-r), round(p.PosY-r), round(p.PosX+r), round(p.PosY+r));
  end;
end;

end.
