unit UTeam;

// TEAM is a group of players that are in a track playing.
// It can have a "Team Intelligence" that will guide the players into the team
// goals

interface

uses Contnrs, SysUtils, Math,
     UPlayer, UGoal;

type
  TTeam = class(TObject)
  private
    // Track I am playing in:
    fTrack: TObject;
    // Player actually "selected":
    fSelectedPlayer: TPlayer;
    // The team members are imiagining futures, please call later
    fImaginig: boolean;
    // Witch things motivate this team (affect all its player):
    fGoals: TObjectList;
    fDebugGoals: boolean;
    // Property getters
    procedure SetImagining(v: boolean);
    function  GetPlayer(i: integer): TPlayer;
    function  GetPlayerCount: integer;
    procedure SetSelectedPlayer(v: TPlayer);
    function  GetGoalCount: integer;
  protected
    function  GetGoal(m: integer): TGoal;
  public
    // List of al the players on track (as TObjects):
    fPlayers:   TObjectList;
    constructor Create;
    destructor  Destroy; override;
    procedure   AddPlayer(v: TPlayer);
    procedure   DelPlayer(v: TPlayer; DoFree: boolean);
    // Do things on all your team members (players):
    procedure   ResetFutures;
    procedure   ResetState;
    procedure   AddGoal(v: TGoal);
    procedure   DelGoal(g: integer);
    procedure   ScoreGoals(FireAt: TGoalFire);
    procedure   DrawGoals;
    procedure   TakeAStep(phase: integer);
    procedure   TakeFirstStep;
    procedure   TakeRandomStep;
    procedure   TakeIntelligentStep;
    procedure   Duplicate;
    procedure   DeleteNosized;
    procedure   Draw;
    procedure   DrawOptions;
    property DebugGoals:   boolean read fDebugGoals   write fDebugGoals;
    property Goal[m: integer]: TGoal read GetGoal;
    property GoalCount: integer read GetGoalCount;
    property SelectedPlayer: TPlayer read fSelectedPlayer write SetSelectedPlayer;
    property Player[i: integer]: TPlayer read GetPlayer;
    property PlayerCount: integer read GetPlayerCount;
    property Track: TObject read fTrack write fTrack;
    property Imagining: boolean read fImaginig write SetImagining;
  end;

implementation

uses UTrack, UTrack2D, UListOfFutures, UPlayer2D;


// **************************
// ** ADMINISTRATIVE STUFF **
// **************************

constructor TTeam.Create;
begin
  inherited;
  fPlayers:= TObjectList.create;
  fPlayers.OwnsObjects:= false;
  fGoals:= TObjectList.create;
  fGoals.OwnsObjects:= false;
  fSelectedPlayer:= nil;
  fImaginig:= false;
  DebugGoals:= false;
end;

destructor TTeam.destroy;
begin
  while fPlayers.Count > 0 do begin
    TPlayer(fPlayers.Items[0]).Free;
    fPlayers.Delete(0);
  end;
  fPlayers.Free;
  while fGoals.Count > 0 do begin
    TGoal(fGoals.Items[0]).Free;
    fGoals.Delete(0);
  end;
  fGoals.Free;
  inherited;
end;


// ********************
// ** THINKING STUFF **
// ********************

// Make the intial random step for this new future:
procedure TTeam.TakeFirstStep;
begin
  TakeAStep(0);
end;

// Make another future's random step:
procedure TTeam.TakeRandomStep;
begin
  TakeAStep(1);
end;

// Make the real -not imagining a future- intelligent step:
procedure TTeam.TakeIntelligentStep;
begin
  TakeAStep(2);
end;

// Taking first step in a future is quite similar with taking the next steps
// or doing the final real step, so the 3 procedures use this general one:
//   phase = 0 : Initial future step.
//   phase = 1 : Following future's steps.
//   phase = 2 : Real step (not imagining futures).
procedure TTeam.TakeAStep(phase: integer);
var p: integer;
begin
  for p:= 0 to PlayerCount-1 do begin
    // Take a decision (first, random or intelligent decision):
    // Note: Take a decision start by storing old state values.
    case phase of
      0: Player[p].TakeFirstDecision;
      1: Player[p].TakeRandomDecision;
      2: Player[p].TakeIntelligentDecision;
    end;
    // Simulate a delta time so you update your internal state:
    Player[p].Simulate;
    // Debugging  (future step or final decision):
    case phase of
      0,1: if Player[p].DebugTraces then Player[p].DrawFuturePoint;
      2:   if Player[p].DebugOptions then Player[p].DrawOptions;
    end;
  end;
end;


// Use team and player's goals to score all players' future:
procedure TTeam.ScoreGoals(FireAt: TGoalFire);
var
  p, g, i: integer;
  Coop, Score, Reduce: double; // -1 compite, +1 = coooperate
begin
  // Each player on the team have to sum goal scores:
  for p:= 0 to PlayerCount-1 do begin
    // Score (and draw) personal player's goals first:
    Player[p].ScoreGoals(FireAt);
    // Add the team's goals' scoring to this player:
    for g:= 0 to GoalCount-1 do begin
      Goal[g].ScoreGoal(Player[p], FireAt);
      if (Goal[g].Score <> 0) or (Goal[g].Reduce <> 1) then begin
        // If goal is of type "cooperative" or "competitive", this score is added
        // or taken to all the other players in the team (a half of it, it should
        // be a coef from -1 to compite, 0 for individualist, up to +1 coopertive):
        if (Goal[g].GoalType <> gtIndividualist) then begin
          // Coef to use to define how much coopertive is it:
          if (Goal[g].GoalType = gtCompetitive) then begin
            // Competitive: Other players loose points and get angry!:
            Coop:= -2;
          end else begin
            // Cooperative: Other playes win points and are happy:
            Coop:= +2;
          end;
          // So scoring and reduction coef. to be applied to all others is:
          Score:=  Coop * Goal[g].Score;
          Reduce:= Power(Goal[g].Reduce+0.000000000001, Coop);
          // Apply it to all others players:
          for i:= 0 to PlayerCount-1 do
            if (i<>p) then
              Player[i].State.AddScore(Score, Reduce);
        end;
      end;
    end;
  end;
end;

// Use team and player's goals to score all players' future:
procedure TTeam.DrawGoals;
var g: integer;
begin
  // If a goal doesn't draw itself with each player then I draw it now:
  if DebugGoals then begin
    // Team goals:
    for g:= 0 to GoalCount-1 do
      if (not Goal[g].Drawable) then
        if (Imagining = Goal[g].DrawOnImagining) then
          Goal[g].NeutralDraw;
  end;
  // Players can have "neutral" goals that work as team goals:
  if (PlayerCount > 0) then begin
    for g:= 0 to Player[0].GoalCount-1 do begin
      if ((Player[0].DebugGoals) and (not Player[0].Goal[g].Drawable)) or
         (Player[0].Goal[0].AlwaysDraw) then begin
        Player[0].Goal[g].NeutralDraw;
      end;
    end;
  end;
end;


// *********************
// ** EVOLUTION STUFF **
// *********************

// Ask players to duplicate if all is ok:
procedure TTeam.Duplicate;
var p: integer;
begin
  for p:= 0 to PlayerCount-1 do
    if Player[p].CanDuplicate then
      Player[p].Clone;
end;

// Dead players loose size, if it get to 1, delete it:
procedure TTeam.DeleteNosized;
var p: integer;
begin
  for p:= PlayerCount-1 downto 0 do
    if (Player[p].Size = Player[p].MinSize) then
      DelPlayer(Player[p], true);
end;



// *******************
// ** DRAWING STUFF **
// *******************

procedure TTeam.Draw;
var p: integer;
begin
  for p:= 0 to PlayerCount-1 do
    Player[p].Draw;
end;

procedure TTeam.DrawOptions;
var p: integer;
begin
  for p:= 0 to PlayerCount-1 do
    Player[p].DrawOptions;
end;


// **********************
// ** MANAGING PLAYERS **
// **********************

// Accesing a member player:
function TTeam.GetPlayer(i: integer): TPlayer;
begin
  result:= TPlayer(FPlayers[i]);
end;

// Get number of players in the team:
function TTeam.GetPlayerCount: integer;
begin
  result:= FPlayers.Count;
end;

// Add a player to the team:
procedure TTeam.AddPlayer(v: TPlayer);
begin
  if fPlayers.IndexOf(v) = -1 then begin
    fPlayers.Add(v);
    v.Team:= Self;
    SelectedPlayer:= v;
    TTrack(Track).RefreshPlayerList;
  end;
end;

// Delete a player from the team:
procedure TTeam.DelPlayer(v: TPlayer; DoFree: boolean);
var p: integer;
begin
  p:= fPlayers.IndexOf(v);
  if (p <> -1) then begin
    // Avoid having a null selected player
    if (SelectedPlayer = v) then
      fSelectedPlayer:= nil;
    // Delete player:
    if DoFree then
      Player[p].free;
    fPlayers.Delete(p);
    // Chose another selected pplayer_
    if not Assigned(fSelectedPlayer) and (PlayerCount>0) then
      fSelectedPlayer:= Player[0];
    // Track be notified:
    TTrack(Track).RefreshPlayerList;
  end;
end;

// Witch player is selected by the user?
procedure TTeam.SetSelectedPlayer(v: TPlayer);
begin
  if fPlayers.IndexOf(v) = -1 then
    raise Exception.create('Selected player must be a team member.');
  fSelectedPlayer:= v;
end;


// *****************
// ** GOALS STUFF **
// *****************

procedure TTeam.AddGoal(v: TGoal);
begin
  fGoals.Add(v);
end;

procedure TTeam.DelGoal(g: integer);
begin
  if (g>=0) and (g<GoalCount) then
    fGoals.Delete(g);
end;

function TTeam.GetGoalCount: integer;
begin
  result:= fGoals.count;
end;

function TTeam.GetGoal(m: integer): TGoal;
begin
  result:= TGoal(fGoals[m]);
end;


// *************************************
// ** ASK ALL YOUR PLAYERS TO DO THIS **
// *************************************

procedure TTeam.SetImagining(v: boolean);
var i: integer;
begin
  // When a team comes back to reality, after thinking a future, goals must be fired:
  if not v then
    ScoreGoals(gfEnd);
  fImaginig:= v;
  for i:= 0 to PlayerCount-1 do
    Player[i].Imagining:= v;
end;

procedure TTeam.ResetFutures;
var i: integer;
begin
  // Reset players futures:
  for i:= 0 to PlayerCount-1 do
    Player[i].ResetFutures;
  // Reset goal imaginary state:
  for i:= 0 to GoalCount-1 do
    Goal[i].ResetImaginaryState;
end;

procedure TTeam.ResetState;
var
  p: integer;
begin
  for p:= 0 to PlayerCount-1 do
    Player[p].ResetState;
end;



end.
